#include "../common/cleanup.h"
#include "../common/const.h"
#include "../common/global.h"
#include "../common/tls.h"

#include <arpa/inet.h>
#include <csignal>
#include <cstring>
#include <iostream>
#include <netinet/in.h>
#include <sys/socket.h>
#include <unistd.h>
#include <wolfssl/options.h>
#include <wolfssl/ssl.h>

WOLFSSL_CTX *ctx;
int sockfd;

int main() {
  std::signal(SIGTERM, safe_exit);
  std::signal(SIGINT, safe_exit);
  std::signal(SIGQUIT, safe_exit);
  std::signal(SIGHUP, safe_exit);

  // error signal handlers
  signal(SIGSEGV, safe_exit);

  // Initialize wolfSSL library
  wolfSSL_Init();

  ctx = create_context(false);

  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    std::cerr << "Socket creation failed: " << strerror(errno) << std::endl;
    return 1;
  }

  std::cout << "Socket created successfully (fd: " << sockfd << ")"
            << std::endl;

  sockaddr_in serv_addr{};
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = htons(PORT);
  inet_pton(AF_INET, "127.0.0.1", &serv_addr.sin_addr);

  if (connect(sockfd, (sockaddr *)&serv_addr, sizeof(serv_addr)) != 0) {
    std::cerr << "Connection failed: " << strerror(errno) << std::endl;
    return 1;
  }

  WOLFSSL *ssl = wolfSSL_new(ctx);
  wolfSSL_set_fd(ssl, sockfd);

  int result = wolfSSL_connect(ssl);
  if (result != WOLFSSL_SUCCESS) {
    int ssl_error = wolfSSL_get_error(ssl, result);
    std::cerr << "wolfSSL_connect failed with error code: " << ssl_error
              << std::endl;
    return 1;
  }

  std::cout << "SSL connection established successfully" << std::endl;

  handle_connection(ssl);

  wolfSSL_shutdown(ssl);
  wolfSSL_free(ssl);
  close(sockfd);
  wolfSSL_CTX_free(ctx);
  wolfSSL_Cleanup();
  return 0;
}
