#include <arpa/inet.h>
#include <cstring>
#include <ifaddrs.h>
#include <net/if.h>
#include <string>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <unistd.h>

std::string getInterfaceIP(const std::string &interfaceName) {
  int sockfd;
  struct ifreq ifr;

  // Create socket
  sockfd = socket(AF_INET, SOCK_DGRAM, 0);
  if (sockfd < 0) {
    return nullptr;
  }

  // Set interface name
  strncpy(ifr.ifr_name, interfaceName.c_str(), IFNAMSIZ - 1);
  ifr.ifr_name[IFNAMSIZ - 1] = '\0';

  // Get IP address
  if (ioctl(sockfd, SIOCGIFADDR, &ifr) < 0) {
    close(sockfd);
    return nullptr;
  }

  close(sockfd);

  // Convert to string
  struct sockaddr_in *addr = (struct sockaddr_in *)&ifr.ifr_addr;
  return std::string(inet_ntoa(addr->sin_addr));
}

int get_interface_index(const char *interface_name) {
  struct ifaddrs *ifaddr, *ifa;
  int interface_index = -1;

  if (getifaddrs(&ifaddr) == -1) {
    return -1;
  }

  for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
    if (ifa->ifa_addr == NULL)
      continue;

    if (strcmp(ifa->ifa_name, interface_name) == 0) {
      interface_index = if_nametoindex(interface_name);
      break;
    }
  }

  freeifaddrs(ifaddr);
  return interface_index;
}