#include "../common/cleanup.h"
#include "../common/const.h"
#include "IF_functions.h"
#include <arpa/inet.h>
#include <cerrno>
#include <cstring>
#include <ifaddrs.h>
#include <iostream>
#include <linux/if_packet.h>
#include <net/if.h>
#include <netinet/if_ether.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <string>
#include <sys/socket.h>
#include <unistd.h>
#include <wolfssl/options.h>
#include <wolfssl/ssl.h>

// this is max size standard Ipv4/6 packet can be plus 1 for message type
// what in understand in practise it is much lower around 1500 bytes
#define IP_PACKET_BUFFER_SIZE 65536 + 1

void process_incoming_ip_packet(unsigned char *buffer, int packet_size,
                                WOLFSSL *ssl) {
  buffer[0] = IP_PACKET_TYPE; // Set message type
  int bytes_sent = wolfSSL_write(ssl, buffer, IP_PACKET_BUFFER_SIZE);
  if (bytes_sent <= 0) {
    int ssl_error = wolfSSL_get_error(ssl, bytes_sent);
    std::cerr << "wolfSSL_write failed with error: " << ssl_error << std::endl;
  }
}

int listen_for_ip_packets(const std::string interface_name, WOLFSSL *ssl) {
  int raw_ip_scoketfd;
  unsigned char buffer[IP_PACKET_BUFFER_SIZE];

  struct sockaddr_ll saddr;
  socklen_t saddr_len = sizeof(saddr);

  // Create raw socket for packet capture
  raw_ip_scoketfd = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_IP));
  if (raw_ip_scoketfd < 0) {
    perror("Failed to create raw socket");
    std::cerr << "Make sure you're running as root!" << std::endl;
    safe_exit(EACCES);
  }

  // Get interface index
  int if_index = get_interface_index(interface_name.c_str());
  if (if_index < 0) {
    std::cerr << "Interface " << interface_name << " not found!" << std::endl;
    close(raw_ip_scoketfd);
    return 1;
  }

  // Bind socket to specific interface
  memset(&saddr, 0, sizeof(saddr));
  saddr.sll_family = AF_PACKET;
  saddr.sll_protocol = htons(ETH_P_IP);
  saddr.sll_ifindex = if_index;

  if (bind(raw_ip_scoketfd, (struct sockaddr *)&saddr, sizeof(saddr)) < 0) {
    perror("Failed to bind to interface");
    close(raw_ip_scoketfd);
    return 1;
  }

  std::cout << "Listening for IP packets on interface: " << interface_name
            << " (index: " << if_index << ")" << std::endl;

  // Main packet capture loop
  while (true) {
    // read and leave first byte for message type
    int packet_size =
        recvfrom(raw_ip_scoketfd, buffer + 1, IP_PACKET_BUFFER_SIZE - 1, 0,
                 (struct sockaddr *)&saddr, &saddr_len);

    if (packet_size < 0) {
      perror("Error receiving packet");
      continue;
    }

    // Only process IP packets (skip non-IP traffic)
    if (packet_size >= sizeof(struct ethhdr) + sizeof(struct iphdr)) {
      process_incoming_ip_packet(buffer, packet_size, ssl);
    }
  }

  close(raw_ip_scoketfd);
  return 0;
}
